//  Copyright 2021 Kakao Corp.
//
//  Licensed under the Apache License, Version 2.0 (the "License");
//  you may not use this file except in compliance with the License.
//  You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
//  Unless required by applicable law or agreed to in writing, software
//  distributed under the License is distributed on an "AS IS" BASIS,
//  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//  See the License for the specific language governing permissions and
//  limitations under the License.

import UIKit
import KakaoPartnerSDKFriend
import KakaoSDKCommon

class PartnerSDKFriendPickerSettingViewController : UIViewController {
    
    var safeAreaLayoutGuide: UILayoutGuide!
    var tableView: UITableView!
    var bottomView: UIView!
    var cancelBtn: UIButton!
    var okBtn: UIButton!
    var viewModel: PartnerSDKPickerSettingViewModel = PartnerSDKFriendPickerSettingViewModel()
    var toastLabel = UILabel(frame: CGRect(x: 0, y: 0, width: 300, height: 35))
    
    override func loadView() {
        super.loadView()
        
        self.safeAreaLayoutGuide = view.layoutMarginsGuide
    
        self.tableView = UITableView(frame: self.view.frame)
        self.tableView.dataSource = self
        self.tableView.delegate = self
        self.tableView.separatorStyle = .none
        self.tableView.register(PartnerSDKFriendPickerSettingCell.self, forCellReuseIdentifier: "cell")
        self.tableView.backgroundColor = .white
        self.view.addSubview(self.tableView)

        self.bottomView = UIView(frame: .zero)
        self.view.addSubview(self.bottomView)

        self.cancelBtn = UIButton(frame: .zero)
        self.cancelBtn.setTitle("취소", for: .normal)
        self.cancelBtn.titleLabel?.font = UIFont.systemFont(ofSize: 16, weight: .bold)
        self.cancelBtn.backgroundColor = .red
        self.cancelBtn.setTitleColor(.black, for: .normal)
        self.cancelBtn.addTarget(self, action: #selector(onCancelBtn), for: .touchUpInside)

        self.okBtn = UIButton(frame: .zero)
        self.okBtn.setTitle("피커 생성", for: .normal)
        self.okBtn.titleLabel?.font = UIFont.systemFont(ofSize: 16, weight: .bold)
        self.okBtn.backgroundColor = .green
        self.okBtn.setTitleColor(.black, for: .normal)
        self.okBtn.addTarget(self, action: #selector(onOkBtn), for: .touchUpInside)

        self.bottomView.addSubview(self.cancelBtn)
        self.bottomView.addSubview(self.okBtn)
        
        self.view.backgroundColor = .white
    }
    
    public override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        self.tableView.translatesAutoresizingMaskIntoConstraints = false
        self.tableView.topAnchor.constraint(equalTo: self.safeAreaLayoutGuide.topAnchor).isActive = true
        self.tableView.leftAnchor.constraint(equalTo: self.view.leftAnchor).isActive = true
        self.tableView.bottomAnchor.constraint(equalTo: self.bottomView.topAnchor).isActive = true
        self.tableView.rightAnchor.constraint(equalTo: self.view.rightAnchor).isActive = true

        self.bottomView.translatesAutoresizingMaskIntoConstraints = false
        self.bottomView.topAnchor.constraint(equalTo: self.tableView.bottomAnchor).isActive = true
        self.bottomView.leftAnchor.constraint(equalTo: self.view.leftAnchor).isActive = true
        self.bottomView.bottomAnchor.constraint(equalTo: self.safeAreaLayoutGuide.bottomAnchor).isActive = true
        self.bottomView.rightAnchor.constraint(equalTo: self.view.rightAnchor).isActive = true
        self.bottomView.heightAnchor.constraint(equalToConstant: 60).isActive = true

        self.cancelBtn.translatesAutoresizingMaskIntoConstraints = false
        self.cancelBtn.topAnchor.constraint(equalTo: self.bottomView.topAnchor).isActive = true
        self.cancelBtn.leftAnchor.constraint(equalTo: self.bottomView.leftAnchor).isActive = true
        self.cancelBtn.bottomAnchor.constraint(equalTo: self.bottomView.bottomAnchor).isActive = true
        self.cancelBtn.widthAnchor.constraint(equalTo: self.bottomView.widthAnchor, multiplier: 0.5).isActive = true

        self.okBtn.translatesAutoresizingMaskIntoConstraints = false
        self.okBtn.topAnchor.constraint(equalTo: self.bottomView.topAnchor).isActive = true
        self.okBtn.rightAnchor.constraint(equalTo: self.bottomView.rightAnchor).isActive = true
        self.okBtn.bottomAnchor.constraint(equalTo: self.bottomView.bottomAnchor).isActive = true
        self.okBtn.widthAnchor.constraint(equalTo: self.bottomView.widthAnchor, multiplier: 0.5).isActive = true
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)
        toastLabel.removeFromSuperview()
    }
    
    @objc func onCancelBtn() {
        dismiss(animated: true, completion: nil)
    }
    
    @objc func onOkBtn() {
        viewModel.printSnapShot()

        if let vm = viewModel as? PartnerSDKFriendPickerSettingViewModel {
            let param = PickerFriendRequestParams(title: vm.pickerTitle,
                                            serviceTypeFilter: vm.serviceTypeFilter,
                                            friendFilter: vm.friendFilter,
                                            countryCodeFilters: vm.countryCodeFilters,
                                            usingOsFilter: vm.usingOsFilter,
                                            friendOrder: vm.friendOrder,
                                            viewAppearance: vm.viewAppearance,
                                            orientation: vm.orientation,
                                            enableSearch: vm.isSearchFriendEnabled,
                                            enableIndex: vm.isIndexViewEnabled,
                                            showMyProfile: vm.isMyProfileEnabled,
                                            showFavorite: vm.isFavoriteEnabled,
                                            disableSelectOptions: vm.disableSelectOptions,
                                            displayAllProfile: vm.displayAllProfile,
                                            showPickedFriend: vm.isSelectedFriendsEnabled,
                                            maxPickableCount: vm.maxPickableCount,
                                            minPickableCount: vm.minPickableCount)
            
            let completion : (SelectedUsers?, Error?) -> Void = { selectedUsers, error in
                if let error = error {
                    self.showToast(message: "\(error)")
                }
                else {
                    if let selectedUsers = selectedUsers {
                        self.showToast(message: "\(selectedUsers)")
                    }
                    else {
                        self.showToast(message: "No Friends")
                    }
                }
            }
            
            if vm.isSinglePicker {
                if vm.isPopupPicker {
                    PickerApi.shared.selectFriendPopup(params: param, completion: completion)
                }
                else {
                    PickerApi.shared.selectFriend(params: param, completion: completion)
                }
            }
            else {
                if vm.isPopupPicker {
                    PickerApi.shared.selectFriendsPopup(params: param, completion: completion)
                }
                else {
                    PickerApi.shared.selectFriends(params: param, completion: completion)
                }
            }
        }
        else if let vm = viewModel as? PartnerSDKChatPickerSettingViewModel {
            
            let param = PickerChatRequestParams(selectionType: vm.selectionType,
                                                title: vm.pickerTitle,
                                                chatFilters: vm.chatFilters,
                                                viewAppearance: vm.viewAppearance,
                                                orientation: vm.orientation,
                                                enableSearch: vm.enableSearch,
                                                disableSelectOptions: vm.disableSelectOptions == nil ? nil : vm.disableSelectOptions,
                                                displayAllProfile: vm.displayAllProfile,
                                                maxPickableCount: vm.maxPickableCount,
                                                minPickableCount: vm.minPickableCount)
            
            let completion : (SelectedUsers?, SelectedChat?, Error?) -> Void = { selectedUsers, selectedChat, error in
                if let error = error {
                    self.showToast(message: "\(error)")
                }
                else {
                    if let selectedUsers = selectedUsers {
                        self.showToast(message: "\(selectedUsers)")
                    }
                    else if let selectedChat = selectedChat {
                        self.showToast(message: "\(selectedChat)")
                    }
                    else {
                        self.showToast(message: "No Friends")
                    }
                }
            }
            
            if vm.isPopupPicker {
               // open poopup
                PickerApi.shared.selectChatPopup(params: param, completion: completion)
            }
            else {
                PickerApi.shared.selectChat(params: param, completion: completion)
            }
        }
        else if let vm = viewModel as? PartnerSDKTapPickerSettingViewModel {
            
            let completion : (SelectedUsers?, SelectedChat?, Error?) -> Void = { selectedUsers, selectedChat, error in
                if let error = error {
                    self.showToast(message: "\(error)")
                }
                else {
                    if let selectedUsers = selectedUsers {
                        self.showToast(message: "\(selectedUsers)")
                    }
                    else if let selectedChat = selectedChat {
                        self.showToast(message: "\(selectedChat)")
                    }
                    else {
                        self.showToast(message: "No Friends")
                    }
                }
            }
            
            let friendRequestParams = PickerTabRequestParams.FriendRequestParams(serviceTypeFilter: vm.serviceTypeFilter,
                                                                                 friendFilter: vm.friendFilter,
                                                                                 countryCodeFilters: vm.countryCodeFilters,
                                                                                 usingOsFilter: vm.usingOsFilter,
                                                                                 friendOrder: vm.friendOrder,
                                                                                 enableIndex: vm.isIndexViewEnabled,
                                                                                 showMyProfile: vm.isMyProfileEnabled,
                                                                                 showFavorite: vm.isFavoriteEnabled,
                                                                                 showPickedFriend: vm.isSelectedFriendsEnabled)
                                                                                 
            let chatRequestParams = PickerTabRequestParams.ChatRequestParams(selectionType:vm.selectionType,
                                                                             chatFilters: vm.chatFilters)
            let tabRequestParams = PickerTabRequestParams(title: vm.pickerTitle,
                                                          viewAppearance: vm.viewAppearance,
                                                          orientation: vm.orientation,
                                                          enableSearch: vm.enableSearch,
                                                          disableSelectOptions: vm.disableSelectOptions,
                                                          displayAllProfile: vm.displayAllProfile,
                                                          maxPickableCount: vm.maxPickableCount,
                                                          minPickableCount: vm.minPickableCount,
                                                          friendParams: friendRequestParams,
                                                          chatParams: chatRequestParams)
            
            if vm.isPopupPicker {
               // open poopup
                PickerApi.shared.selectPopup(params: tabRequestParams, completion: completion)
            }
            else {
                PickerApi.shared.select(params: tabRequestParams, completion: completion)
            }
        }
        else {
            
        }
       
    }
    
    func showToast(message : String) {
        toastLabel.removeFromSuperview()
        toastLabel = UILabel(frame: CGRect(x: 0,
                                               y: 0, width: 300, height: 35))
        toastLabel.backgroundColor = UIColor.black.withAlphaComponent(0.6)
        toastLabel.textColor = UIColor.white
        toastLabel.font = UIFont.systemFont(ofSize: 18.0)
        toastLabel.textAlignment = .center;
        toastLabel.text = message
        toastLabel.alpha = 0.0
        toastLabel.layer.cornerRadius = 10;
        toastLabel.clipsToBounds = true
        toastLabel.numberOfLines = 0
        toastLabel.sizeToFit()

        self.view.addSubview(toastLabel)
        toastLabel.center = CGPoint(x: UIScreen.main.bounds.width/2, y: self.view.bounds.height - toastLabel.bounds.height/2.0 - 30)
        
        UIView.animate(withDuration: 0.1, delay: 0.0, options: .curveLinear, animations: { self.toastLabel.alpha = 1.0 },
            completion: {(isCompleted) in
                DispatchQueue.main.asyncAfter(deadline: .now() + 3.5) {
                    UIView.animate(withDuration: 0.3, delay: 0.0, options: .curveLinear, animations: { self.toastLabel.alpha = 0.0 },
                        completion: {(isCompleted) in
                            self.toastLabel.removeFromSuperview()
                        }
                    )
                }
            }
        )
    }
    
    override func touchesBegan(_ touches: Set<UITouch>, with event: UIEvent?) {
        super.touchesBegan(touches, with: event)
        view.endEditing(true)
     }
}


extension PartnerSDKFriendPickerSettingViewController: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        view.endEditing(true)
    }
}

extension PartnerSDKFriendPickerSettingViewController: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return viewModel.numberOfItems
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if let cell = tableView.dequeueReusableCell(withIdentifier: "cell") as? PartnerSDKFriendPickerSettingCell {
            cell.item = viewModel.items[indexPath.item]
            cell.backgroundColor = .white
            cell.selectionStyle = .none
            cell.update()
            return cell
        }

        return PartnerSDKFriendPickerSettingCell()
    }
    
    public func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 30
    }
}


class PartnerSDKFriendPickerSettingCell : UITableViewCell {
    
    var parentViewController: UIViewController? {
        var parentResponder: UIResponder? = self
        while parentResponder != nil {
            parentResponder = parentResponder!.next
            if let viewController = parentResponder as? UIViewController {
                return viewController
            }
        }
        return nil
    }
    
    var item: Item?
    var checkBox: UIButton?
    var inputBox: UITextField?
    var menuBox: UIButton?
    var multipleCheckBoxCollectionView: UICollectionView?
    
    override func prepareForReuse() {
        super.prepareForReuse()
        multipleCheckBoxCollectionView?.delegate = nil
        multipleCheckBoxCollectionView?.dataSource = nil
        
        for view in contentView.subviews {
            view.removeFromSuperview()
        }
    }
    
    func update() {
        guard let item = item else { return }
        
        let titleLabel = UILabel(frame: .zero)
        titleLabel.text = item.title
        titleLabel.textColor = .black
        titleLabel.sizeToFit()
        titleLabel.font = UIFont.systemFont(ofSize: 16, weight: .semibold)
        self.contentView.addSubview(titleLabel)
        
        titleLabel.translatesAutoresizingMaskIntoConstraints = false
        titleLabel.leftAnchor.constraint(equalTo: self.contentView.leftAnchor, constant: 20).isActive = true
        titleLabel.widthAnchor.constraint(equalToConstant: titleLabel.bounds.width).isActive = true
        titleLabel.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true

        if let it = (item as? CheckBoxItem) {
            checkBox = UIButton(frame: .zero)
            checkBox?.layer.cornerRadius = 15
            checkBox?.layer.masksToBounds = true
            checkBox?.layer.borderWidth = 1.0
            checkBox?.layer.borderColor = UIColor.black.cgColor
            checkBox?.setBackgroundImage(it.value == true ? UIImage(named: "checked") : nil, for: .normal)
            checkBox?.addTarget(self, action: #selector(handleCheckBox), for: .touchUpInside)

            self.contentView.addSubview(checkBox!)
            checkBox?.translatesAutoresizingMaskIntoConstraints = false
            checkBox?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: -30).isActive = true
            checkBox?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            checkBox?.widthAnchor.constraint(equalToConstant: 29).isActive = true
            checkBox?.heightAnchor.constraint(equalToConstant: 29).isActive = true
        }
        else if let it = (item as? InputBoxItem) {
            inputBox = UITextField(frame: .zero)
            if it.type == .numeric { inputBox?.keyboardType = .numberPad}
            inputBox?.addDoneButtonOnKeyboard()
            inputBox?.backgroundColor = .white
            inputBox?.layer.cornerRadius = 5
            inputBox?.layer.masksToBounds = true
            inputBox?.text = it.value
            inputBox?.textAlignment = .center
            inputBox?.autocorrectionType = .no
            inputBox?.textColor = .black
            inputBox?.backgroundColor = #colorLiteral(red: 0.8977650404, green: 0.8979157209, blue: 0.8977451921, alpha: 1)
            inputBox?.attributedPlaceholder = NSAttributedString(string: it.placeHolder ?? "",
                                                                attributes: [NSAttributedString.Key.foregroundColor: UIColor.darkGray])
            inputBox?.addTarget(self, action: #selector(textFieldDidChange), for: .editingChanged)
            
            inputBox?.layer.borderWidth = 0.5
            inputBox?.layer.borderColor = UIColor.black.cgColor
            inputBox?.delegate = self

            self.contentView.addSubview(inputBox!)
            inputBox?.translatesAutoresizingMaskIntoConstraints = false
            inputBox?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: -30).isActive = true
            inputBox?.leftAnchor.constraint(equalTo: titleLabel.rightAnchor, constant: 30).isActive = true
            inputBox?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            inputBox?.heightAnchor.constraint(equalToConstant: 29).isActive = true
        }
        else if let it = (item as? MenuBoxItem) {

            menuBox = UIButton(frame: .zero)
            menuBox?.layer.borderWidth = 0.5
            menuBox?.layer.borderColor = UIColor.black.cgColor
            menuBox?.layer.cornerRadius = 5
            menuBox?.layer.masksToBounds = true
            menuBox?.setTitleColor(UIColor.black, for: .normal)
            menuBox?.setTitle(it.value, for: .normal)
            menuBox?.backgroundColor = #colorLiteral(red: 0.8977650404, green: 0.8979157209, blue: 0.8977451921, alpha: 1)
            
            menuBox?.addTarget(self, action: #selector(handleMenuBox), for: .touchUpInside)
            
            self.contentView.addSubview(menuBox!)
            menuBox?.translatesAutoresizingMaskIntoConstraints = false
            menuBox?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: -30).isActive = true
            menuBox?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            menuBox?.widthAnchor.constraint(equalToConstant: 150).isActive = true
            menuBox?.heightAnchor.constraint(equalToConstant: 29).isActive = true
        }
        else if (item as? MultipleCheckBoxItem) != nil {
            
            let layout = UICollectionViewFlowLayout()
            layout.scrollDirection = .horizontal
            
            multipleCheckBoxCollectionView = UICollectionView(frame: .zero, collectionViewLayout: layout)
            multipleCheckBoxCollectionView?.dataSource = self
            multipleCheckBoxCollectionView?.delegate = self
            multipleCheckBoxCollectionView?.backgroundColor = #colorLiteral(red: 0.8977650404, green: 0.8979157209, blue: 0.8977451921, alpha: 1)
            multipleCheckBoxCollectionView?.contentInset = UIEdgeInsets(top: 0, left: 5, bottom: 0, right: 0)
            
            multipleCheckBoxCollectionView?.layer.borderWidth = 0.5
            multipleCheckBoxCollectionView?.layer.borderColor = UIColor.black.cgColor
            multipleCheckBoxCollectionView?.layer.cornerRadius = 5
            multipleCheckBoxCollectionView?.layer.masksToBounds = true
            
            multipleCheckBoxCollectionView?.collectionViewLayout = layout
            multipleCheckBoxCollectionView?.register(PartnerSDKFriendPickerSettingMultiCheckCell.self, forCellWithReuseIdentifier: "multiCheckCell")
            
            self.contentView.addSubview(multipleCheckBoxCollectionView!)
            multipleCheckBoxCollectionView?.translatesAutoresizingMaskIntoConstraints = false
            multipleCheckBoxCollectionView?.rightAnchor.constraint(equalTo: self.contentView.rightAnchor, constant: 0).isActive = true
            multipleCheckBoxCollectionView?.leftAnchor.constraint(equalTo: titleLabel.rightAnchor, constant: 10).isActive = true
            multipleCheckBoxCollectionView?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
            multipleCheckBoxCollectionView?.heightAnchor.constraint(equalToConstant: 29).isActive = true
            multipleCheckBoxCollectionView?.showsHorizontalScrollIndicator = false
            layout.itemSize = CGSize(width: 80, height: 29)
        }
        
    }
    
    @objc func handleCheckBox() {
        if let it = item as? CheckBoxItem {
            let enable = it.action?() ?? false
            checkBox?.setBackgroundImage(enable == true ? UIImage(named: "checked") : nil, for: .normal)
            it.value = enable
        }
    }
    
    @objc func handleMenuBox() {
        if let it = item as? MenuBoxItem {
            if let vc = self.parentViewController {
                
                let alert = UIAlertController(title: nil, message: nil, preferredStyle: .actionSheet)
                
                for menu in it.menu {
                    alert.addAction(UIAlertAction(title: menu, style: .default) { _ in
                        self.menuBox?.setTitle(menu, for: .normal)
                        let selectedMenu = it.action?(menu)
                        it.value = selectedMenu
                    })
                }
                
                let cancel = UIAlertAction(title: "CANCEL", style: .destructive, handler: nil)
                alert.addAction(cancel)
                
                if UIDevice.current.userInterfaceIdiom == .pad {
                    if let popoverController = alert.popoverPresentationController {
                        popoverController.sourceView = vc.view
                        popoverController.sourceRect = CGRect(x: vc.view.bounds.midX, y: vc.view.bounds.midY, width: 0, height: 0)
                        popoverController.permittedArrowDirections = []
                        vc.present(alert, animated: true, completion: nil)
                    }
                }
                else {
                    vc.present(alert, animated: true, completion: {
                        print("completion block")
                    })
                }
            }
        }
    }
}

extension PartnerSDKFriendPickerSettingCell: UITextFieldDelegate {
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        if let it = item as? InputBoxItem {
            it.action?(textField.text)
            it.value = textField.text
        }
    }
    
    func textFieldShouldReturn(_ textField: UITextField) -> Bool {
        textField.endEditing(true)
        return true
    }
    
    @objc func textFieldDidChange() {
        
        if let it = item as? InputBoxItem {
            let text = inputBox?.text
            it.action?(text)
            it.value = text
        }
    }
}


extension PartnerSDKFriendPickerSettingCell: UICollectionViewDelegate, UICollectionViewDataSource {
    func collectionView(_ collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return (item as? MultipleCheckBoxItem)?.values.count ?? 0
    }
    
    func collectionView(_ collectionView: UICollectionView, cellForItemAt indexPath: IndexPath) -> UICollectionViewCell {
        guard let items = item as? MultipleCheckBoxItem else {
            return PartnerSDKFriendPickerSettingMultiCheckCell()
        }

        if let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "multiCheckCell", for: indexPath) as? PartnerSDKFriendPickerSettingMultiCheckCell {
            cell.item = items.values[indexPath.item]
            cell.backgroundColor = .white
            cell.create()
            return cell
        }

        return PartnerSDKFriendPickerSettingMultiCheckCell()
    }
    
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        guard let cell = collectionView.dequeueReusableCell(withReuseIdentifier: "multiCheckCell", for: indexPath) as? PartnerSDKFriendPickerSettingMultiCheckCell else {
            return
        }
 
        guard let items = item as? MultipleCheckBoxItem else {
            return
        }

        let item = items.values[indexPath.item]
        if let key = item.keys.first {
            items.values[indexPath.item] = [key: !((item.values.first) ?? true)]
            cell.item = items.values[indexPath.item]
        }
        collectionView.reloadData()
        
        _ = items.action!(cell.item)
    }
    
}


class PartnerSDKFriendPickerSettingMultiCheckCell : UICollectionViewCell {
    
    var item: [String: Bool] = [:]
    var titleLabel: UILabel?
    
    
    override func prepareForReuse() {
        super.prepareForReuse()
        item = [:]
        titleLabel = nil
        
        for view in contentView.subviews {
            view.removeFromSuperview()
        }
    }
    
    func create() {
        
        if titleLabel == nil {
            titleLabel = UILabel(frame: .zero)
            titleLabel?.text = item.keys.first

            titleLabel?.textAlignment = .center
            titleLabel?.sizeToFit()
            titleLabel?.font = UIFont.systemFont(ofSize: 12, weight: .semibold)
            self.contentView.addSubview(titleLabel!)
            
            titleLabel?.translatesAutoresizingMaskIntoConstraints = false
         //   titleLabel?.widthAnchor.constraint(equalToConstant: titleLabel!.frame.width).isActive = true
            titleLabel?.centerXAnchor.constraint(equalTo:self.contentView.centerXAnchor).isActive = true
            titleLabel?.centerYAnchor.constraint(equalTo:self.contentView.centerYAnchor).isActive = true
        }

        titleLabel?.textColor = .black
        self.contentView.backgroundColor = #colorLiteral(red: 1, green: 1, blue: 1, alpha: 1)
        
        if item.values.first == true {
            self.contentView.backgroundColor = #colorLiteral(red: 0, green: 0, blue: 0, alpha: 1)
            titleLabel?.textColor = #colorLiteral(red: 1, green: 1, blue: 1, alpha: 1)
        }
    }

}

class PartnerSDKPickerSettingViewModel {
    
    var items: [Item] = []
    
    var numberOfItems: Int {
        return items.count
    }
    
    func printSnapShot() {
        
    }
}


// ViewModel
class PartnerSDKFriendPickerSettingViewModel: PartnerSDKPickerSettingViewModel{
    
    let DEFAULT_TITLE = "친구 선택"
    
    // Setting Value
    var isSinglePicker = false
    var isPopupPicker = false
    var pickerTitle: String?// = "카카오톡 친구 선택"
    var serviceTypeFilter: PickerServiceTypeFilter?
    var friendFilter: PickerFriendFilter?
    var countryCodeFilters: [String]?
    var usingOsFilter: PickerUsingOsFilter?
    var friendOrder: PickerFriendOrder?
    var viewAppearance: ViewAppearance?
    var orientation: PickerOrientation?
    var isSearchFriendEnabled: Bool = true
    var isIndexViewEnabled: Bool = true
    var isMyProfileEnabled: Bool = true
    var isFavoriteEnabled: Bool = true
    var disableSelectOptions: [DisableSelectOption]?
    var maxPickableCount: Int = 30
    var minPickableCount: Int = 1
    var isSelectedFriendsEnabled = true
    var displayAllProfile: Bool = false
    ///
    override init() {
        super.init()
        items.append(CheckBoxItem(title: "싱글 피커", action: {
            self.isSinglePicker = !self.isSinglePicker
            return self.isSinglePicker
        }))
        items.append(CheckBoxItem(title: "팝업 피커", action: {
            self.isPopupPicker = !self.isPopupPicker
            return self.isPopupPicker
        }))
        items.append(InputBoxItem(title: "타이틀", placeHolder: DEFAULT_TITLE, action: { pickerTitle in
            
            if let title = pickerTitle {
                if title.isEmpty {
                    self.pickerTitle = nil
                }
                else {
                    self.pickerTitle = title
                }
            }
            else {
                self.pickerTitle = nil
            }

        }))
        
        items.append(MenuBoxItem(title: "친구 정렬", menu: ["NULL","AGE"], action: { type in
            switch type {
            case "AGE":
                self.friendOrder = .age
            default:
                self.friendOrder = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "친구 타입", menu: ["NULL","TALK","STORY","TALKSTORY"], action: { type in
            switch type {
            case "TALK":
                self.serviceTypeFilter = .talk
            case "STORY":
                self.serviceTypeFilter = .story
            case "TALKSTORY":
                self.serviceTypeFilter = .talkstory
            default:
                self.serviceTypeFilter = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "친구 필터", menu: ["NULL","REGISTERED","INVITABLE","NONE"], action: { type in
            switch type {
            case "REGISTERED":
                self.friendFilter = .registered
            case "INVITABLE":
                self.friendFilter = .invitable
            case "NONE":
                self.friendFilter = PickerFriendFilter.none
            default:
                self.friendFilter = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "Light/Dark 모드", menu: ["NULL","AUTO","LIGHT","DARK"], action: { type in
            switch type {
            case "AUTO":
                self.viewAppearance = .auto
            case "LIGHT":
                self.viewAppearance = .light
            case "DARK":
                self.viewAppearance = .dark
            default:
                self.viewAppearance = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(InputBoxItem(title: "국가 코드", placeHolder: "kr,jp,us,....", action: { countryCodeFilters in
            self.countryCodeFilters = countryCodeFilters?.split(separator: ",").map{it in String(it)}
        }))
        
        
        items.append(MenuBoxItem(title: "친구 디바이스 OS", menu: ["NULL","ALL","ANDROID","iOS"], action: { type in
            switch type {
            case "ALL":
                self.usingOsFilter = .all
            case "ANDROID":
                self.usingOsFilter = .android
            case "iOS":
                self.usingOsFilter = .ios
            default:
                self.usingOsFilter = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "피커 Orientation", menu: ["NULL","AUTO","PORTRAIT","LANDSCAPE"], action: { type in
            switch type {
            case "AUTO":
                self.orientation = .auto
            case "PORTRAIT":
                self.orientation = .portrait
            case "LANDSCAPE":
                self.orientation = .landscape
            default:
                self.orientation = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(CheckBoxItem(title: "검색 허용", value: true, action: {
            self.isSearchFriendEnabled = !self.isSearchFriendEnabled
            return self.isSearchFriendEnabled
        }))
        
        items.append(CheckBoxItem(title: "인덱스 허용", value: true, action: {
            self.isIndexViewEnabled = !self.isIndexViewEnabled
            return self.isIndexViewEnabled
        }))
        
        items.append(CheckBoxItem(title: "내 프로필 보기", value: true, action: {
            self.isMyProfileEnabled = !self.isMyProfileEnabled
            return self.isMyProfileEnabled
        }))

        items.append(CheckBoxItem(title: "내 즐겨찾기 보기", value: true, action: {
            self.isFavoriteEnabled = !self.isFavoriteEnabled
            return self.isFavoriteEnabled
        }))
        
        items.append(MultipleCheckBoxItem(title: "친구 비활성화 옵션", values: [["MsgBlocked":false], ["Registered":false], ["UnRegistered":false], ], action: { data in

            if self.disableSelectOptions == nil {
                self.disableSelectOptions = []
            }
            
            switch data.keys.first {
                case "MsgBlocked":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.msgBlocked) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .msgBlocked}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.msgBlocked))
                    }
                case "Registered":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.registered) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .registered}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.registered))
                    }
                case "UnRegistered":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.unRegistered) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .unRegistered}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.unRegistered))
                    }
                default:
                    print("error")
            }
            
            if self.disableSelectOptions?.count == 0 {
                self.disableSelectOptions = nil
            }
            
            return true
        }))
        
        items.append(InputBoxItem(title: "최대 선택 수", placeHolder: "30", type: .numeric,action: { count in
            if let count = count {
                if count.isEmpty {
                    self.maxPickableCount = 30
                }
                else {
                    self.maxPickableCount = Int(count) ??  self.maxPickableCount
                }
            }
            else {
                self.maxPickableCount = 30
            }
        }))
        
        items.append(InputBoxItem(title: "최소 선택 수", placeHolder: "1",type: .numeric, action: { count in
            if let count = count {
                if count.isEmpty {
                    self.minPickableCount = 1
                }
                else {
                    self.minPickableCount = Int(count) ??  self.minPickableCount
                }
            }
            else {
                self.minPickableCount = 1
            }
        }))
        
        items.append(CheckBoxItem(title: "선택된 친구 보기", value: true, action: {
            self.isSelectedFriendsEnabled = !self.isSelectedFriendsEnabled
            return self.isSelectedFriendsEnabled
        }))
        
        items.append(CheckBoxItem(title: "모든 데이터 보기(권한 있는 앱만 적용 됨)", value: false, action: {
            self.displayAllProfile = !self.displayAllProfile
            return self.displayAllProfile
        }))
    }
    
    override func printSnapShot() {
        print("*************** SNAPSHOT *********************")
        print("isSinglePicker: \(isSinglePicker), isPopupPicker: \(isPopupPicker)")
        print("pickerTitle: \(String(describing: pickerTitle)), serviceTypeFilter: \(String(describing: serviceTypeFilter))")
        print("friendOrder: \(String(describing: friendOrder))")
        print("friendFilter: \(String(describing: friendFilter)), viewApperance: \(String(describing: viewAppearance))")
        print("countryCodeFilters: \(String(describing: countryCodeFilters)), usingOsFilter: \(String(describing: usingOsFilter))")
        print("orientation: \(String(describing: orientation)), isSearchFriendEnabled: \(isSearchFriendEnabled)")
        print("isIndexViewEnabled: \(isIndexViewEnabled)")
        print("isMyProfileEnabled: \(isMyProfileEnabled)")
        print("isFavoriteEnabled: \(isFavoriteEnabled), disableSelectOptions: \(String(describing: disableSelectOptions?.enumerated().map{$0.element.reason})))")
        print("maxPickableCount: \(maxPickableCount), minPickableCount: \(minPickableCount)")
        print("isSelectedFriendsEnabled: \(isSelectedFriendsEnabled), displayAllProfile: \(displayAllProfile)")
        print("**********************************************")
    }
}


class PartnerSDKChatPickerSettingViewModel: PartnerSDKPickerSettingViewModel {
    
    let DEFAULT_TITLE = "친구 선택"
    
    // Setting Value
    var isPopupPicker = false
    var pickerTitle: String?
    var chatFilters: [PickerChatFilter] = [.regular]
    var selectionType: PickerChatSelectionType = .chat
    var viewAppearance: ViewAppearance?
    var orientation: PickerOrientation?
    var enableSearch: Bool = true
    var disableSelectOptions: [DisableSelectOption]?
    var maxPickableCount: Int = 30
    var minPickableCount: Int = 1
    var displayAllProfile: Bool = false
    ///
    override init() {
        super.init()
        
        items.append(CheckBoxItem(title: "팝업 피커", action: {
            self.isPopupPicker = !self.isPopupPicker
            return self.isPopupPicker
        }))
        items.append(CheckBoxItem(title: "검색 허용", value: true, action: {
            self.enableSearch = !self.enableSearch
            return self.enableSearch
        }))
        items.append(InputBoxItem(title: "타이틀", placeHolder: DEFAULT_TITLE, action: { pickerTitle in
            
            if let title = pickerTitle {
                if title.isEmpty {
                    self.pickerTitle = nil
                }
                else {
                    self.pickerTitle = title
                }
            }
            else {
                self.pickerTitle = nil
            }

        }))
        
        items.append(MenuBoxItem(title: "챗방/챗맴버 피커 타입", menu: ["chat","chatMember"], action: { type in
            switch type {
            case "chat":
                self.selectionType = .chat
            case "chatMember":
                self.selectionType = .chatMember
            default:
                self.selectionType = .chat
            }
            return type ?? "chat"
        }))
        
        items.append(MenuBoxItem(title: "Light/Dark 모드", menu: ["NULL","AUTO","LIGHT","DARK"], action: { type in
            switch type {
            case "AUTO":
                self.viewAppearance = .auto
            case "LIGHT":
                self.viewAppearance = .light
            case "DARK":
                self.viewAppearance = .dark
            default:
                self.viewAppearance = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "피커 Orientation", menu: ["NULL","AUTO","PORTRAIT","LANDSCAPE"], action: { type in
            switch type {
            case "AUTO":
                self.orientation = .auto
            case "PORTRAIT":
                self.orientation = .portrait
            case "LANDSCAPE":
                self.orientation = .landscape
            default:
                self.orientation = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(InputBoxItem(title: "최대 선택 수", placeHolder: "30", type: .numeric,action: { count in
            if let count = count {
                if count.isEmpty {
                    self.maxPickableCount = 30
                }
                else {
                    self.maxPickableCount = Int(count) ??  self.maxPickableCount
                }
            }
            else {
                self.maxPickableCount = 30
            }
        }))
        
        items.append(InputBoxItem(title: "최소 선택 수", placeHolder: "1",type: .numeric, action: { count in
            if let count = count {
                if count.isEmpty {
                    self.minPickableCount = 1
                }
                else {
                    self.minPickableCount = Int(count) ??  self.minPickableCount
                }
            }
            else {
                self.minPickableCount = 1
            }
        }))
        
        items.append(CheckBoxItem(title: "모든 데이터 보기(권한 있는 앱만 적용 됨)", value: false, action: {
            self.displayAllProfile = !self.displayAllProfile
            return self.displayAllProfile
        }))

        items.append(MultipleCheckBoxItem(title: "챗 필터", values: [["regular":true], ["open":false], ["multi":false], ["direct":false]], action: { data in

            switch data.keys.first {
                case "regular":
                    if self.chatFilters.contains(.regular) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .regular}
                    }
                    else {
                        self.chatFilters.append(.regular)
                    }
                case "open":
                    if self.chatFilters.contains(.open) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .open}
                    }
                    else {
                        self.chatFilters.append(.open)
                    }
                case "multi":
                    if self.chatFilters.contains(.multi) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .multi}
                    }
                    else {
                        self.chatFilters.append(.multi)
                    }
                case "direct":
                    if self.chatFilters.contains(.direct) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .direct}
                    }
                    else {
                        self.chatFilters.append(.direct)
                    }
                default:
                    print("error")
            }
            return true
        }))
        
        items.append(MultipleCheckBoxItem(title: "친구 비활성화 옵션", values: [["MsgBlocked":false], ["Registered":false], ["UnRegistered":false], ["NotFriend":false]], action: { data in

            if self.disableSelectOptions == nil {
                self.disableSelectOptions = []
            }
            
            switch data.keys.first {
                case "MsgBlocked":
                    if (self.disableSelectOptions!.map({ $0.reason}).contains(.msgBlocked)) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .msgBlocked}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.msgBlocked))
                    }
                case "Registered":
                    if self.disableSelectOptions!.map({ $0.reason}).contains(.registered) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .registered}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.registered))
                    }
                case "UnRegistered":
                    if self.disableSelectOptions!.map({ $0.reason}).contains(.unRegistered) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .unRegistered}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.unRegistered))
                    }
                case "NotFriend":
                    if self.disableSelectOptions!.map({ $0.reason}).contains(.notFriend) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .notFriend}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.notFriend))
                    }
                default:
                    print("error")
            }
            
            if self.disableSelectOptions?.count == 0 {
                self.disableSelectOptions = nil
            }
            
            return true
        }))
    }
    
    override func printSnapShot() {
        print("*************** SNAPSHOT *********************")
        print("isPopupPicker: \(isPopupPicker)")
        print("pickerTitle: \(String(describing: pickerTitle)), chatFilters: \(chatFilters)")
        print("selectionType: \(String(describing: selectionType))")
        print("viewApperance: \(String(describing: viewAppearance))")
        print("orientation: \(String(describing: orientation)), enableSearch: \(enableSearch)")
        print("disableSelectOptions: \(String(describing: disableSelectOptions?.enumerated().map{$0.element.reason}))")
        print("maxPickableCount: \(maxPickableCount), minPickableCount: \(minPickableCount)")
        print("displayAllProfile: \(displayAllProfile)")
        print("**********************************************")
    }
}

class PartnerSDKTapPickerSettingViewModel: PartnerSDKPickerSettingViewModel {
    
    let DEFAULT_TITLE = "친구 선택"
    
    // Setting Value
    var isPopupPicker = false
    var pickerTitle: String?
    var viewAppearance: ViewAppearance?
    var orientation: PickerOrientation?
    var enableSearch: Bool = true
    var disableSelectOptions: [DisableSelectOption]?
    var maxPickableCount: Int = 30
    var minPickableCount: Int = 1
    var displayAllProfile: Bool = false
    
    // friend only
    var friendOrder: PickerFriendOrder?
    var serviceTypeFilter: PickerServiceTypeFilter?
    var friendFilter: PickerFriendFilter?
    var countryCodeFilters: [String]?
    var usingOsFilter: PickerUsingOsFilter?
    var isIndexViewEnabled: Bool = true
    var isMyProfileEnabled: Bool = true
    var isFavoriteEnabled: Bool = true
    var isSelectedFriendsEnabled = true
    
    // chat only
    var chatFilters: [PickerChatFilter] = [.regular]
    var selectionType: PickerChatSelectionType = .chat
    
    ///
    override init() {
        super.init()
        
        items.append(CheckBoxItem(title: "팝업 피커", action: {
            self.isPopupPicker = !self.isPopupPicker
            return self.isPopupPicker
        }))
        items.append(CheckBoxItem(title: "검색 허용", value: true, action: {
            self.enableSearch = !self.enableSearch
            return self.enableSearch
        }))
        items.append(InputBoxItem(title: "타이틀", placeHolder: DEFAULT_TITLE, action: { pickerTitle in
            
            if let title = pickerTitle {
                if title.isEmpty {
                    self.pickerTitle = nil
                }
                else {
                    self.pickerTitle = title
                }
            }
            else {
                self.pickerTitle = nil
            }

        }))
        items.append(MenuBoxItem(title: "친구 정렬", menu: ["NULL","AGE"], action: { type in
            switch type {
            case "AGE":
                self.friendOrder = .age
            default:
                self.friendOrder = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "친구 타입", menu: ["NULL","TALK","STORY","TALKSTORY"], action: { type in
            switch type {
            case "TALK":
                self.serviceTypeFilter = .talk
            case "STORY":
                self.serviceTypeFilter = .story
            case "TALKSTORY":
                self.serviceTypeFilter = .talkstory
            default:
                self.serviceTypeFilter = nil
            }
            return type ?? "NULL"
        }))
        items.append(InputBoxItem(title: "국가 코드", placeHolder: "kr,jp,us,....", action: { countryCodeFilters in
            self.countryCodeFilters = countryCodeFilters?.split(separator: ",").map{it in String(it)}
        }))
        
        
        items.append(MenuBoxItem(title: "친구 디바이스 OS", menu: ["NULL","ALL","ANDROID","iOS"], action: { type in
            switch type {
            case "ALL":
                self.usingOsFilter = .all
            case "ANDROID":
                self.usingOsFilter = .android
            case "iOS":
                self.usingOsFilter = .ios
            default:
                self.usingOsFilter = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(CheckBoxItem(title: "인덱스 허용", value: true, action: {
            self.isIndexViewEnabled = !self.isIndexViewEnabled
            return self.isIndexViewEnabled
        }))
        
        items.append(CheckBoxItem(title: "내 프로필 보기", value: true, action: {
            self.isMyProfileEnabled = !self.isMyProfileEnabled
            return self.isMyProfileEnabled
        }))

        items.append(CheckBoxItem(title: "내 즐겨찾기 보기", value: true, action: {
            self.isFavoriteEnabled = !self.isFavoriteEnabled
            return self.isFavoriteEnabled
        }))
        
        items.append(CheckBoxItem(title: "선택된 친구 보기", value: true, action: {
            self.isSelectedFriendsEnabled = !self.isSelectedFriendsEnabled
            return self.isSelectedFriendsEnabled
        }))
        
        items.append(MenuBoxItem(title: "친구 필터", menu: ["NULL","REGISTERED","INVITABLE","NONE"], action: { type in
            switch type {
            case "REGISTERED":
                self.friendFilter = .registered
            case "INVITABLE":
                self.friendFilter = .invitable
            case "NONE":
                self.friendFilter = PickerFriendFilter.none
            default:
                self.friendFilter = nil
            }
            return type ?? "NULL"
        }))
        items.append(MenuBoxItem(title: "챗방/챗맴버 피커 타입", menu: ["chat","chatMember"], action: { type in
            switch type {
            case "chat":
                self.selectionType = .chat
            case "chatMember":
                self.selectionType = .chatMember
            default:
                self.selectionType = .chat
            }
            return type ?? "chat"
        }))
        
        items.append(MenuBoxItem(title: "Light/Dark 모드", menu: ["NULL","AUTO","LIGHT","DARK"], action: { type in
            switch type {
            case "AUTO":
                self.viewAppearance = .auto
            case "LIGHT":
                self.viewAppearance = .light
            case "DARK":
                self.viewAppearance = .dark
            default:
                self.viewAppearance = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(MenuBoxItem(title: "피커 Orientation", menu: ["NULL","AUTO","PORTRAIT","LANDSCAPE"], action: { type in
            switch type {
            case "AUTO":
                self.orientation = .auto
            case "PORTRAIT":
                self.orientation = .portrait
            case "LANDSCAPE":
                self.orientation = .landscape
            default:
                self.orientation = nil
            }
            return type ?? "NULL"
        }))
        
        items.append(InputBoxItem(title: "최대 선택 수", placeHolder: "30", type: .numeric,action: { count in
            if let count = count {
                if count.isEmpty {
                    self.maxPickableCount = 30
                }
                else {
                    self.maxPickableCount = Int(count) ??  self.maxPickableCount
                }
            }
            else {
                self.maxPickableCount = 30
            }
        }))
        
        items.append(InputBoxItem(title: "최소 선택 수", placeHolder: "1",type: .numeric, action: { count in
            if let count = count {
                if count.isEmpty {
                    self.minPickableCount = 1
                }
                else {
                    self.minPickableCount = Int(count) ??  self.minPickableCount
                }
            }
            else {
                self.minPickableCount = 1
            }
        }))
        
        items.append(CheckBoxItem(title: "모든 데이터 보기(권한 있는 앱만 적용 됨)", value: false, action: {
            self.displayAllProfile = !self.displayAllProfile
            return self.displayAllProfile
        }))

        items.append(MultipleCheckBoxItem(title: "챗 필터", values: [["regular":true], ["open":false], ["multi":false], ["direct":false]], action: { data in

            switch data.keys.first {
                case "regular":
                    if self.chatFilters.contains(.regular) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .regular}
                    }
                    else {
                        self.chatFilters.append(.regular)
                    }
                case "open":
                    if self.chatFilters.contains(.open) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .open}
                    }
                    else {
                        self.chatFilters.append(.open)
                    }
                case "multi":
                    if self.chatFilters.contains(.multi) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .multi}
                    }
                    else {
                        self.chatFilters.append(.multi)
                    }
                case "direct":
                    if self.chatFilters.contains(.direct) {
                        self.chatFilters = self.chatFilters.filter{ $0 != .direct}
                    }
                    else {
                        self.chatFilters.append(.direct)
                    }
                default:
                    print("error")
            }
            return true
        }))
        
        items.append(MultipleCheckBoxItem(title: "친구 비활성화 옵션", values: [["MsgBlocked":false], ["Registered":false], ["UnRegistered":false], ["NotFriend":false]], action: { data in

            if self.disableSelectOptions == nil {
                self.disableSelectOptions = []
            }
            
            switch data.keys.first {
                case "MsgBlocked":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.msgBlocked) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .msgBlocked}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.msgBlocked))
                    }
                case "Registered":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.registered) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .registered}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.registered))
                    }
                case "UnRegistered":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.unRegistered) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .unRegistered}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.unRegistered))
                    }
                case "NotFriend":
                    if self.disableSelectOptions!.map({ $0.reason }).contains(.notFriend) {
                        self.disableSelectOptions = self.disableSelectOptions!.filter{ $0.reason != .notFriend}
                    }
                    else {
                        self.disableSelectOptions!.append(DisableSelectOption(reason:.notFriend))
                    }
                default:
                    print("error")
            }
            
            if self.disableSelectOptions?.count == 0 {
                self.disableSelectOptions = nil
            }
            
            return true
        }))
    }
    
    override func printSnapShot() {
        print("*************** SNAPSHOT *********************")
        print("isPopupPicker: \(isPopupPicker)")
        print("pickerTitle: \(String(describing: pickerTitle)), chatFilters: \(chatFilters)")
        print("selectionType: \(String(describing: selectionType)) friendOrder: \(String(describing: friendOrder))")
        print("viewApperance: \(String(describing: viewAppearance)) serviceTypeFilter: \(String(describing: serviceTypeFilter))")
        print("orientation: \(String(describing: orientation)), enableSearch: \(enableSearch)")
        print("disableSelectOptions: \(String(describing: disableSelectOptions?.enumerated().map{$0.element.reason})), friendFilter: \(String(describing: friendFilter))")
        print("maxPickableCount: \(maxPickableCount), minPickableCount: \(minPickableCount)")
        print("displayAllProfile: \(displayAllProfile) usingOsFilter: \(String(describing: usingOsFilter))")
        print("isIndexViewEnabled: \(isIndexViewEnabled) isMyProfileEnabled: \(isMyProfileEnabled)")
        print("isFavoriteEnabled: \(isFavoriteEnabled) isSelectedFriendsEnabled: \(isSelectedFriendsEnabled)")
        print("**********************************************")
    }
}
// Model
class Item {
    let title: String
    init(title: String) {
        self.title = title
    }
}

class CheckBoxItem: Item {
    var value : Bool?
    var action : (() -> Bool)?
    
    init(title: String, value: Bool? = false, action: @escaping (() -> Bool)) {
        super.init(title: title)
        
        self.value = value
        self.action = action
    }
}

class MultipleCheckBoxItem: Item {

    var values : [[String : Bool]] = []
    var action : (([String : Bool]) -> Bool)?
    
    init(title: String, values: [[String : Bool]], action: @escaping (([String : Bool]) -> Bool)) {
        super.init(title: title)
        
        self.values = values
        self.action = action
    }
}

class InputBoxItem: Item {
    
    enum InputBoxType{
        case string
        case numeric
    }
    
    var value: String?
    var placeHolder: String?
    var action: ((String?) -> Void)?
    var type: InputBoxType = .string
    
    init(title: String, placeHolder: String?, value: String? = "", type: InputBoxType = .string, action: @escaping ((String?) -> Void)) {
        super.init(title: title)
        
        self.value = value
        self.placeHolder = placeHolder
        self.action = action
        self.type = type
    }
}

class MenuBoxItem: Item {
    var value: String?
    var menu: [String] = []
    var action : ((String?) -> String)?
    
    init(title: String, value: String? = "NULL", menu: [String], action: @escaping ((String?) -> String)) {
        super.init(title: title)
        
        self.value = value
        self.menu = menu
        self.action = action
    }
}


extension UITextField {
    func addDoneButtonOnKeyboard() {
        let doneToolbar: UIToolbar = UIToolbar(frame: CGRect.init(x: 0, y: 0, width: UIScreen.main.bounds.width, height: 50))
        doneToolbar.barStyle = .default
        let flexSpace = UIBarButtonItem(barButtonSystemItem: .flexibleSpace, target: nil, action: nil)
        let done: UIBarButtonItem = UIBarButtonItem(title: "완료", style: .done, target: self, action: #selector(self.doneButtonAction))
        let items = [flexSpace, done]
        doneToolbar.items = items
        doneToolbar.sizeToFit()
        self.inputAccessoryView = doneToolbar
    }
    @objc func doneButtonAction() {
        self.resignFirstResponder()
    }
}
